//+-----------------------------------------------------------------------+
//|                                          ATM_MasterCandle_IB_PBv3.mq4 |
//| Identifies the following candle formations :                          |
//| Inside Bars - candle is within or touching previous candle extreme    |
//| Master Inside Bars - a series of consecutive IB's                     |
//| PinBars - candle shadow is greater than defined ratio                 |
//| V3  Removed color key and bar timer
//+-----------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 6
#property indicator_color1 Gold
#property indicator_color2 Gold
#property indicator_width1 5
#property indicator_width2 5
#property indicator_color3 Lime
#property indicator_color4 Lime
#property indicator_width3 0
#property indicator_width4 0
#property indicator_color5 Black
#property indicator_color6 Black
#property indicator_width5 0
#property indicator_width6 0

static string   Version             ="1.0";

double MasterIB_High[], MasterIB_Low[];
double IB_High[], IB_Low[];
double PinBar_High[], PinBar_Low[];
int BarCount;

extern string TimeFrame            = "current time frame";
extern int    MasterIB_Extra_Pips  =0;
extern string a_                   ="Default BarsBeforePaint";
extern string b_                   ="M1, M2, M3, or M4 default 5 bars";
extern string c_                   ="M5 or greater     default 4 bars";
extern string d_                   ="Set to 0 to use defaults";
extern int    MasterIB_Minimum_Bars=5;
extern color  MasterIB_Color       =Gold;
extern string e_                   ="Set width to 0 to disable";
extern int    MasterIB_Width       =5;

extern color  IB_Color             =Lime;
extern string f_                   ="Set width to 0 to disable";
extern int    IB_Width             =5;

extern int    Minimum_Nose_Ratio   =60;
extern int    Maximum_Body_Ratio   =40;
extern color  PinBar_Color         =Black;
extern string g_                   ="Set width to 0 to disable";
extern string g1_                  ="Set width to -size to show as star";
extern string g2_                  ="eg -1=small, -3=larger";
extern int    PinBar_Width         =0;
extern string   Indicator_Name      ="MasterCandlePlus";


int      timeFrame;
bool     calculateValue;
bool     returnBars;
string   indicatorFileName;

int init()
  {

   IndicatorBuffers(6);
   SetIndexBuffer( 0, MasterIB_High );  SetIndexStyle( 0, DRAW_HISTOGRAM, 0, MasterIB_Width, MasterIB_Color );
   SetIndexBuffer( 1, MasterIB_Low );   SetIndexStyle( 1, DRAW_HISTOGRAM, 0, MasterIB_Width, MasterIB_Color );
   SetIndexBuffer( 2, IB_High );        SetIndexStyle( 2, DRAW_HISTOGRAM, 0, IB_Width, IB_Color );
   SetIndexBuffer( 3, IB_Low );         SetIndexStyle( 3, DRAW_HISTOGRAM, 0, IB_Width, IB_Color );
   if( PinBar_Width>0 )
     {
      SetIndexBuffer( 4, PinBar_High ); SetIndexStyle( 4, DRAW_HISTOGRAM, 0, PinBar_Width, PinBar_Color );
      SetIndexBuffer( 5, PinBar_Low );  SetIndexStyle( 5, DRAW_HISTOGRAM, 0, PinBar_Width, PinBar_Color );
     }
   else if( PinBar_Width<0 )
     {
      SetIndexBuffer( 4, PinBar_High );  SetIndexArrow( 4, 171 ); SetIndexStyle( 4, DRAW_ARROW, 0, MathAbs( PinBar_Width ), PinBar_Color );
      SetIndexBuffer( 5, PinBar_Low );   SetIndexArrow( 5, 171 ); SetIndexStyle( 5, DRAW_ARROW, 0, MathAbs( PinBar_Width ), PinBar_Color );
      
     }

         //
         //
         //
         //
         //
                  
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
         //
         //
         //
         //
         //

   return(0);
  }

int deinit()
  {
   int _ObjectsTotal=ObjectsTotal();
   for( int _Object=_ObjectsTotal; _Object>=0; _Object-- )
     {
      string _ObjectName=ObjectName( _Object );
      if( StringSubstr( _ObjectName, 0, StringLen( Indicator_Name )+2 )=="["+Indicator_Name+"]" ) ObjectDelete( _ObjectName );
     }    
   return(0);
  }


double work[][2];
#define _high 0
#define _low  1
int start()
{
   double bar_length, noseup_length, nosedn_length, body_length;
   int shift,bar,counted_bars=IndicatorCounted();

      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { MasterIB_High[0] = limit+1; return(0); }
         if( MasterIB_Minimum_Bars==0 )
         {   
            if( Period() < 5 )  MasterIB_Minimum_Bars=5;
            else                MasterIB_Minimum_Bars=4;
         }    

   
   double HighestHigh  = work[Bars-limit-1][_high];
   double LowestLow    = work[Bars-limit-1][_low];  
   if (calculateValue || timeFrame==Period())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      
      for( shift=limit; shift>0; shift-- )
      {
//--- Pin Bar
      if( PinBar_Width!=0 )
        {
         bar_length=High[shift]-Low[shift];
         if( bar_length==0 ) bar_length=Point;
         noseup_length=MathMin( Open[shift], Close[shift] )-Low[shift];
         nosedn_length=High[shift]-MathMax( Open[shift], Close[shift] );
         body_length=MathAbs( Open[shift]-Close[shift] );
         if(
            noseup_length/bar_length>Minimum_Nose_Ratio/100.0
            &&
            body_length/bar_length<Maximum_Body_Ratio/100.0
            &&
            Low[shift+1]-Low[shift]>=bar_length/3.0
           )
           {
            if( PinBar_Width>0 )
              {
               if( Open[shift]<Close[shift] ) PinBar_High[shift]=Open[shift];
               else                           PinBar_High[shift]=Close[shift];
               PinBar_Low[shift]=Low[shift];
              }
            else if( PinBar_Width<0 )
              {
               PinBar_Low[shift]=Low[shift]-Point;
              }
           }
         if(
            nosedn_length/bar_length>Minimum_Nose_Ratio/100.0
            &&
            body_length/bar_length<Maximum_Body_Ratio/100.0
            &&
            High[shift]-High[shift+1]>=bar_length/3.0
           )
           {
            if( PinBar_Width>0 )
              {
               PinBar_High[shift]=High[shift];
               if( Open[shift]>Close[shift] ) PinBar_Low[shift]=Open[shift];
               else                           PinBar_Low[shift]=Close[shift];
              }
            else if( PinBar_Width<0 )
              {
               PinBar_High[shift]=High[shift]+Point;
              }
           }
        }
//--- Inside Bar
      if( IB_Width!=0 )
        {
         if(
            ( High[shift+1]>=High[shift] && Low[shift+1]<Low[shift] )
            ||
            ( High[shift+1]>High[shift] && Low[shift+1]<=Low[shift] )
           )
           {
		       IB_High[shift]=High[shift];
		       IB_Low[shift] =Low[shift];
		     }
		  }
//--- Master Inside Bar
      if( MasterIB_Width!=0 )
        {
         if( High[shift]<=HighestHigh && Low[shift]>=LowestLow ) 
           {
             BarCount++;  
           }
         else
           {
            HighestHigh=High[shift];
            LowestLow  = Low[shift];
            BarCount=0;
           }
         if( BarCount>=MasterIB_Minimum_Bars )
           {        
            for( bar=0; bar<BarCount; bar++ )   
              {
               MasterIB_High[shift+bar]=HighestHigh+( MasterIB_Extra_Pips*Point ); 
	            MasterIB_Low[shift+bar] =LowestLow  -( MasterIB_Extra_Pips*Point );  
	           }  
           }
        }
        work[Bars-shift-1][_high] = HighestHigh; 
        work[Bars-shift-1][_low]  = LowestLow; 
     }
                                                       ObjectMove( "["+Indicator_Name+"] CandleTime", 0, Time[0], Close[0] );
   return(0);
   }
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (int i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         MasterIB_High[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",MasterIB_Extra_Pips,"","","","",MasterIB_Minimum_Bars,MasterIB_Color,"",MasterIB_Width,IB_Color,"",IB_Width,Minimum_Nose_Ratio,Maximum_Body_Ratio,PinBar_Color,"","","",PinBar_Width,Indicator_Name,0,y);
         MasterIB_Low[i]  = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",MasterIB_Extra_Pips,"","","","",MasterIB_Minimum_Bars,MasterIB_Color,"",MasterIB_Width,IB_Color,"",IB_Width,Minimum_Nose_Ratio,Maximum_Body_Ratio,PinBar_Color,"","","",PinBar_Width,Indicator_Name,1,y);
         IB_High[i]       = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",MasterIB_Extra_Pips,"","","","",MasterIB_Minimum_Bars,MasterIB_Color,"",MasterIB_Width,IB_Color,"",IB_Width,Minimum_Nose_Ratio,Maximum_Body_Ratio,PinBar_Color,"","","",PinBar_Width,Indicator_Name,2,y);
         IB_Low[i]        = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",MasterIB_Extra_Pips,"","","","",MasterIB_Minimum_Bars,MasterIB_Color,"",MasterIB_Width,IB_Color,"",IB_Width,Minimum_Nose_Ratio,Maximum_Body_Ratio,PinBar_Color,"","","",PinBar_Width,Indicator_Name,3,y);
         PinBar_High[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",MasterIB_Extra_Pips,"","","","",MasterIB_Minimum_Bars,MasterIB_Color,"",MasterIB_Width,IB_Color,"",IB_Width,Minimum_Nose_Ratio,Maximum_Body_Ratio,PinBar_Color,"","","",PinBar_Width,Indicator_Name,4,y);
         PinBar_Low[i]    = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",MasterIB_Extra_Pips,"","","","",MasterIB_Minimum_Bars,MasterIB_Color,"",MasterIB_Width,IB_Color,"",IB_Width,Minimum_Nose_Ratio,Maximum_Body_Ratio,PinBar_Color,"","","",PinBar_Width,Indicator_Name,5,y);
   }
   return(0);         
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}